from ._api import API_VERSION
from .analyze import Analyze
from .endpoint import Endpoint
from .info import Info
from .root_certs_raw import RootCertsRaw
from .status_codes import StatusCodes

__all__ = ["API_VERSION", "Analyze", "Endpoint", "Info", "RootCertsRaw", "StatusCodes"]



from dataclasses import dataclass
from typing import List

from .simulation import SimulationData


@dataclass
class SimDetailsData:
    """Dataclass for Simulation object lists.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#simdetails
    """

    results: List[SimulationData]
    """Instances of Simulation."""



from dacite import from_dict

from ..data.host import HostData
from ._api import _Api


class Analyze(_Api):
    """Invoke assessment and check progress.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#invoke-assessment-and-check-progress
    """

    async def get(self, host: str, **kwargs) -> HostData:
        """Analyze host.

        :param host: Hostname to analyze
        :key publish: Set to "on" if assessment results should be published on the public results boards; optional, defaults
                      to "off".
        :key startNew: If set to "on" then cached assessment results are ignored and a new assessment is started. However,
                       if there's already an assessment in progress, its status is delivered instead. This parameter should be
                       used only once to initiate a new assessment; further invocations should omit it to avoid causing an
                       assessment loop.
        :key fromCache: Always deliver cached assessment reports if available; optional, defaults to "off". This parameter is
                        intended for API consumers that don't want to wait for assessment results. Can't be used at the same
                        time as the startNew parameter.
        :key maxAge: Maximum report age, in hours, if retrieving from cache (fromCache parameter set).
        :key all: By default this call results only summaries of individual endpoints. If this parameter is set to "on", full
                  information will be returned. If set to "done", full information will be returned only if the assessment is
                  complete (status is READY or ERROR).
        :key ignoreMismatch: Set to "on" to proceed with assessments even when the server certificate doesn't match the
                             assessment hostname. Set to off by default. Please note that this parameter is ignored if a
                             cached report is returned.
        :raises httpx.ConnectTimeout: SSL Labs Servers don't respond.
        :raises httpx.HTTPStatusError: A client or server error response occured.
        :raises httpx.ReadTimeout: SSL Labs Servers don't respond.
        """
        self._verify_kwargs(kwargs.keys(), ["publish", "startNew", "fromCache", "maxAge", "all", "ignoreMismatch"])
        r = await self._call("analyze", host=host, **kwargs)
        return from_dict(data_class=HostData, data=r.json())



from dataclasses import dataclass
from typing import List

from .trust_path import TrustPathData


@dataclass
class CertificateChainData:
    """Dataclass for certificate chain objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#certificatechain
    """

    id: str
    """Certificate chain ID"""

    certIds: List[str]
    """
    List of IDs of each certificate, representing the chain certificates in the order in which they were retrieved from the
    server
    """

    trustPaths: List[TrustPathData]
    """Trust path object"""

    issues: int
    """A number of flags that describe the chain and the problems it has."""

    noSni: bool
    """True for certificate obtained only with No Server Name Indication (SNI)."""



from dataclasses import dataclass


@dataclass
class DrownHostsData:
    """Dataclass for Drown Hosts objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#drownhosts
    """

    ip: str
    """IP address of server that shares same RSA-Key/hostname in its certificate"""

    export: bool
    """True if export cipher suites detected"""

    port: int
    """Port number of the server"""

    special: bool
    """True if vulnerable OpenSSL version detected"""

    sslv2: bool
    """True if SSL v2 is supported"""

    status: str
    """Drown host status"""



from dataclasses import dataclass
from typing import Optional


@dataclass
class TrustData:
    """Dataclass for trust objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#trust
    """

    rootStore: str
    """this field shows the Trust store being used (eg. 'Mozilla')"""

    isTrusted: Optional[bool]
    """True if trusted against above rootStore"""

    trustErrorMessage: Optional[str]
    """Shows the error message if any"""



from dacite import from_dict

from ..data.status_codes import StatusCodesData
from ._api import _Api


class StatusCodes(_Api):
    """Retrieve known status codes.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#retrieve-known-status-codes
    """

    async def get(self) -> StatusCodesData:
        """Retrieve known status codes.

        :raises httpx.ConnectTimeout: SSL Labs Servers don't respond.
        :raises httpx.HTTPStatusError: A client or server error response occured.
        :raises httpx.ReadTimeout: SSL Labs Servers don't respond.
        """
        r = await self._call("getStatusCodes")
        return from_dict(data_class=StatusCodesData, data=r.json())



from dataclasses import dataclass
from typing import Dict


@dataclass
class StatusCodesData:
    """Dataclass for StatusCodes instances.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#statuscodes
    """

    statusDetails: Dict
    """
    A map containing all status details codes and the corresponding English translations. Please note that, once in use, the
    codes will not change, whereas the translations may change at any time.
    """



from dataclasses import dataclass
from typing import List, Optional

from .caa_policy import CaaPolicyData


@dataclass
class CertData:
    """Dataclass for Cert objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#cert
    """

    id: str
    """Certificate ID"""

    subject: str
    """Certificate subject"""

    serialNumber: str
    """Certificate serial number (hex-encoded)"""

    commonNames: List[str]
    """Common names extracted from the subject"""

    altNames: Optional[List[str]]
    """Alternative names"""

    notBefore: int
    """Timestamp before which the certificate is not valid (Unix Timestamp)"""

    notAfter: int
    """Timestamp after which the certificate is not valid (Unix Timestamp)"""

    issuerSubject: str
    """Issuer subject"""

    sigAlg: str
    """Certificate signature algorithm"""

    revocationInfo: int
    """A number that represents revocation information present in the certificate"""

    crlURIs: Optional[List[str]]
    """CRL URIs extracted from the certificate"""

    ocspURIs: Optional[List[str]]
    """OCSP URIs extracted from the certificate"""

    revocationStatus: int
    """A number that describes the revocation status of the certificate"""

    crlRevocationStatus: int
    """Same as revocationStatus, but only for the CRL information (if any)."""

    ocspRevocationStatus: int
    """Same as revocationStatus, but only for the OCSP information (if any)."""

    dnsCaa: Optional[bool]
    """True if CAA is supported else false."""

    caaPolicy: Optional[CaaPolicyData]
    """CAA Policy, Null if CAA is not supported"""

    mustStaple: bool
    """True if stapling is supported else false"""

    sgc: int
    """Server Gated Cryptography support"""

    validationType: Optional[str]
    """E for Extended Validation certificates"""

    issues: Optional[int]
    """list of certificate issues, one bit per issue"""

    sct: bool
    """True if the certificate contains an embedded SCT; false otherwise."""

    sha1Hash: str
    """sha1 hash of the certificate"""

    sha256Hash: str
    """sha256 hash of the certificate"""

    pinSha256: str
    """sha256 hash of the public key"""

    keyAlg: str
    """Key algorithm."""

    keySize: int
    """Key size, in bits appropriate for the key algorithm."""

    keyStrength: int
    """Key strength, in equivalent RSA bits"""

    keyKnownDebianInsecure: Optional[bool]
    """True if debian flaw is found, else false"""

    raw: str
    """PEM-encoded certificate"""



import logging
from abc import ABC
from typing import KeysView, Optional

import httpx

API_VERSION = 3
SSLLABS_URL = f"https://api.ssllabs.com/api/v{API_VERSION}/"


class _Api(ABC):
    """Abstract class to communicate with Qualys SSL Labs Assessment APIs."""

    def __init__(self, client: Optional[httpx.AsyncClient] = None):
        self._logger = logging.getLogger(f"{self.__class__.__module__}.{self.__class__.__name__}")
        self._client = client

    async def _call(self, api_endpoint: str, **kwargs) -> httpx.Response:
        """Invocate API."""
        if self._client:
            r = await self._client.get(f"{SSLLABS_URL}{api_endpoint}", params=kwargs)
        else:
            async with httpx.AsyncClient() as client:
                r = await client.get(f"{SSLLABS_URL}{api_endpoint}", params=kwargs)
        r.raise_for_status()
        return r

    def _verify_kwargs(self, given: KeysView, known: list):
        """Log warning, if an argument is unknown."""
        for arg in given:
            if arg not in known:
                self._logger.warning(
                    "Argument '%s' is not known by the SSL Labs API. It will be send, but the results might be unexpected.",
                    arg)



from dataclasses import dataclass
from typing import Optional

from .endpoint_details import EndpointDetailsData


@dataclass
class EndpointData:
    """Dataclass for endpoint objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#endpoint
    """

    ipAddress: str
    """Endpoint IP address, in IPv4 or IPv6 format."""

    serverName: Optional[str]
    """Server name retrieved via reverse DNS"""

    statusMessage: str
    """Assessment status message; this field will contain 'Ready' if the endpoint assessment was successful."""

    statusDetails: Optional[str]
    """Code of the operation currently in progress"""

    statusDetailsMessage: Optional[str]
    """Description of the operation currently in progress"""

    grade: Optional[str]
    """Possible values: A+, A-, A-F, T (no trust) and M (certificate name mismatch)"""

    gradeTrustIgnored: Optional[str]
    """Grade (as above), if trust issues are ignored"""

    futureGrade: Optional[str]
    """Next grade because of upcoming grading criteria changes, Null if there is no impact on current grade."""

    hasWarnings: Optional[bool]
    """If this endpoint has warnings that might affect the score (e.g., get A- instead of A)."""

    isExceptional: Optional[bool]
    """
    This flag will be raised when an exceptional configuration is encountered. The SSL Labs test will give such sites an A+.
    """

    progress: Optional[int]
    """Assessment progress, which is a value from 0 to 100, and -1 if the assessment has not yet started"""

    duration: Optional[int]
    """Assessment duration, in milliseconds"""

    eta: Optional[int]
    """Estimated time, in seconds, until the completion of the assessment"""

    delegation: int
    """Indicates domain name delegation with and without the www prefix"""

    details: Optional[EndpointDetailsData]
    """
    This field contains an EndpointDetails object. It's not present by default, but can be enabled by using the "all"
    parameter to the analyze API call.
    """



from dataclasses import dataclass
from typing import List, Optional

from .suite import SuiteData


@dataclass
class ProtocolSuitesData:
    """Dataclass for protocol suites objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#protocolsuites
    """

    protocol: int
    """Protocol version."""

    list: List[SuiteData]
    """List of Suite objects"""

    preference: Optional[bool]
    """
    True if the server actively selects cipher suites; if null, we were not able to determine if the server has a preference
    """

    chaCha20Preference: Optional[bool]
    """
    True if the server takes into account client preferences when deciding if to use ChaCha20 suites. null, we were not able
    to determine if the server has a chacha preference.
    """



from dataclasses import dataclass
from typing import List, Optional

from .cert import CertData
from .endpoint import EndpointData


@dataclass
class HostData:
    """Dataclass for host objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#host
    """

    host: str
    """Assessment host, which can be a hostname or an IP address"""

    port: int
    """Assessment port (e.g., 443)"""

    protocol: str
    """Protocol (e.g., HTTP)"""

    isPublic: bool
    """True if this assessment is publicly available (listed on the SSL Labs assessment boards)"""

    status: str
    """Assessment status; possible values: DNS, ERROR, IN_PROGRESS, and READY."""

    statusMessage: Optional[str]
    """Status message in English. When status is ERROR, this field will contain an error message."""

    startTime: int
    """Assessment starting time, in milliseconds since 1970"""

    testTime: Optional[int]
    """Assessment completion time, in milliseconds since 1970"""

    engineVersion: str
    """Assessment engine version (e.g., '1.26.5')"""

    criteriaVersion: str
    """Grading criteria version (e.g., '2009l')"""

    cacheExpiryTime: Optional[int]
    """
    When will the assessment results expire from the cache (typically set only for assessment with errors; otherwise the
    results stay in the cache for as long as there's sufficient room)
    """

    certHostnames: Optional[List[str]]
    """
    The list of certificate hostnames collected from the certificates seen during assessment. The hostnames may not be valid.
    This field is available only if the server certificate doesn't match the requested hostname. In that case, this field
    saves you some time as you don't have to inspect the certificates yourself to find out what valid hostnames might be.
    """

    endpoints: Optional[List[EndpointData]]
    """List of Endpoint objects"""

    certs: Optional[List[CertData]]
    """
    A list of Cert object, representing the chain certificates in the order in which they were retrieved from the server.
    """



from dataclasses import dataclass
from typing import Dict, List, Optional


@dataclass
class HpkpPolicyData:
    """Dataclass for HPKP Policy objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#hpkppolicy
    """

    header: Optional[str]
    """The contents of the HPKP response header, if present"""

    status: str
    """HPKP status"""

    error: Optional[str]
    """Error message, when the policy is invalid"""

    maxAge: Optional[int]
    """The max-age value from the policy"""

    includeSubDomains: Optional[bool]
    """True if the includeSubDomains directive is set; null otherwise"""

    reportUri: Optional[str]
    """The report-uri value from the policy"""

    pins: List[Dict]
    """List of all pins used by the policy"""

    matchedPins: List[Dict]
    """List of pins that match the current configuration"""

    directives: List[Dict]
    """List of raw policy directives (name-value pairs)"""



from dataclasses import dataclass
from typing import List


@dataclass
class InfoData:
    """Dataclass for info objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#info
    """

    engineVersion: str
    """SSL Labs software version as a string (e.g., '1.11.14')"""

    criteriaVersion: str
    """Rating criteria version as a string (e.g., '2009f')"""

    maxAssessments: int
    """The maximum number of concurrent assessments the client is allowed to initiate."""

    currentAssessments: int
    """The number of ongoing assessments submitted by this client."""

    newAssessmentCoolOff: int
    """
    The cool-off period after each new assessment, in milliseconds; you're not allowed to submit a new assessment before
    the cool-off expires, otherwise you'll get a 429.
    """

    messages: List[str]
    """
    A list of messages (strings). Messages can be public (sent to everyone) and private (sent only to the invoking client).
    Private messages are prefixed with '[Private]'.
    """



from dataclasses import dataclass
from typing import List, Optional

from .named_group import NamedGroupData


@dataclass
class NamedGroupsData:
    """Dataclass for NamedGroup object lists.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#namedgroups
    """

    list: List[NamedGroupData]
    """An array of NamedGroup objects"""

    preference: Optional[bool]
    """True if the server has preferred curves that it uses first"""



from dataclasses import dataclass
from typing import Dict, List, Optional


@dataclass
class StaticPkpPolicyData:
    """Dataclass for SPKP Policy objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#staticpkppolicy
    """

    status: str
    """SPKP status"""

    error: Optional[str]
    """Error message, when the policy is invalid"""

    includeSubDomains: Optional[bool]
    """True if the includeSubDomains directive is set else false"""

    reportUri: Optional[str]
    """The report-uri value from the policy"""

    pins: List[Dict]
    """List of all pins used by the policy"""

    matchedPins: List[Dict]
    """List of pins that match the current configuration"""

    forbiddenPins: List[Dict]
    """List of all forbidden pins used by policy"""

    matchedForbiddenPins: List[Dict]
    """List of forbidden pins that match the current configuration"""



from dataclasses import dataclass
from typing import List

from .caa_record import CaaRecordData


@dataclass
class CaaPolicyData:
    """Dataclass for CAA Policy objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#caapolicy
    """

    policyHostname: str
    """hostname where policy is located"""

    caaRecords: List[CaaRecordData]
    """List of supported CAARecord"""



from dacite import from_dict

from ..data.endpoint import EndpointData
from ._api import _Api


class Endpoint(_Api):
    """Retrieve detailed endpoint information.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#retrieve-detailed-endpoint-information
    """

    async def get(self, host: str, s: str, **kwargs) -> EndpointData:
        """Retrieve detailed endpoint information.

        :param host: Hostname to analyze
        :param s: Endpoint IP address
        :key fromCache: Always deliver cached assessment reports if available; optional, defaults to "off". This parameter is
                        intended for API consumers that don't want to wait for assessment results. Can't be used at the same
                        time as the startNew parameter.
        :raises httpx.ConnectTimeout: SSL Labs Servers don't respond.
        :raises httpx.HTTPStatusError: A client or server error response occured.
        :raises httpx.ReadTimeout: SSL Labs Servers don't respond.
        """
        self._verify_kwargs(kwargs.keys(), ["fromCache"])
        r = await self._call("getEndpointData", host=host, s=s, **kwargs)
        return from_dict(data_class=EndpointData, data=r.json())



from dataclasses import dataclass
from typing import Optional


@dataclass
class ProtocolData:
    """Dataclass for protocol objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#protocol
    """

    id: int
    """Protocol version, e.g. 771 for TLS 1.2 (0x0303)"""

    name: str
    """Protocol name SSL/TLS."""

    version: str
    """Protocol version, e.g. 1.2, 1.1 etc"""

    v2SuitesDisabled: Optional[bool]
    """
    Some servers have SSLv2 protocol enabled, but with all SSLv2 cipher suites disabled. In that case, this field is set to
    True.
    """

    q: Optional[int]
    """0 if the protocol is insecure"""



from dataclasses import dataclass
from typing import Optional


@dataclass
class SuiteData:
    """Dataclass for suite objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#suite
    """

    id: int
    """Suite RFC ID"""

    name: str
    """Suite name (e.g., TLS_RSA_WITH_RC4_128_SHA)"""

    cipherStrength: int
    """Suite strength (e.g., 128)"""

    kxType: Optional[str]
    """Key exchange type (e.g., ECDH)"""

    kxStrength: Optional[int]
    """Key exchange strength, in RSA-equivalent bits"""

    dhP: Optional[int]
    """DH params, p component"""

    dhG: Optional[int]
    """DH params, g component"""

    dhYs: Optional[int]
    """DH params, Ys component"""

    namedGroupBits: Optional[int]
    """EC bits"""

    namedGroupId: Optional[int]
    """EC curve ID"""

    namedGroupName: Optional[str]
    """EC curve name"""

    q: Optional[int]
    """Flag for suite insecure or weak. Not present if suite is strong or good"""



from dataclasses import dataclass
from typing import List, Optional

from .trust import TrustData


@dataclass
class TrustPathData:
    """Dataclass for trust path objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#trustpath
    """

    certIds: List[str]
    """List of certificate ID from leaf to root."""

    trust: List[TrustData]
    """Trust object. This object shows info about the trusted certificate by using Mozilla trust store."""

    isPinned: Optional[bool]
    """True if a key is pinned, else false"""

    matchedPins: Optional[int]
    """Number of matched pins with HPKP policy"""

    unmatchedPins: Optional[int]
    """Number of unmatched pins with HPKP policy"""



from dataclasses import dataclass
from typing import List, Optional

from .certificate_chain import CertificateChainData
from .drown_hosts import DrownHostsData
from .hpkp_policy import HpkpPolicyData
from .hsts_policy import HstsPolicyData
from .hsts_preload import HstsPreloadData
from .http_transaction import HttpTransactionData
from .named_groups import NamedGroupsData
from .protocol import ProtocolData
from .protocol_suites import ProtocolSuitesData
from .sim_details import SimDetailsData
from .static_pkp_policy import StaticPkpPolicyData


@dataclass
class EndpointDetailsData:
    """Dataclass for endpoint detail objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#endpointdetails
    """

    hostStartTime: int
    """
    Endpoint assessment starting time, in milliseconds since 1970. This field is useful when test results are retrieved in
    several HTTP invocations. Then, you should check that the hostStartTime value matches the startTime value of the host.
    """

    certChains: List[CertificateChainData]
    """Server Certificate chains"""

    protocols: List[ProtocolData]
    """Supported protocols"""

    suites: Optional[List[ProtocolSuitesData]]
    """Supported cipher suites per protocol"""

    noSniSuites: Optional[ProtocolSuitesData]
    """Cipher suites observed only with client that does not support Server Name Indication (SNI)."""

    namedGroups: Optional[NamedGroupsData]
    """Instance of NamedGroups object."""

    serverSignature: Optional[str]
    """
    Contents of the HTTP Server response header when known. This field could be absent for one of two reasons: 1) the HTTP
    request failed (check httpStatusCode) or 2) there was no Server response header returned.
    """

    prefixDelegation: bool
    """True if this endpoint is reachable via a hostname with the www prefix"""

    nonPrefixDelegation: bool
    """True if this endpoint is reachable via a hostname without the www prefix"""

    vulnBeast: Optional[bool]
    """True if the endpoint is vulnerable to the BEAST attack"""

    renegSupport: Optional[int]
    """This is an integer value that describes the endpoint support for renegotiation."""

    sessionResumption: Optional[int]
    """This is an integer value that describes endpoint support for session resumption."""

    compressionMethods: Optional[int]
    """Integer value that describes supported compression methods"""

    supportsNpn: Optional[bool]
    """True if the server supports NPN"""

    npnProtocols: Optional[str]
    """Space separated list of supported NPN protocols"""

    supportsAlpn: Optional[bool]
    """True if the server supports ALPN"""

    alpnProtocols: Optional[str]
    """Space separated list of supported ALPN protocols"""

    sessionTickets: Optional[int]
    """Indicates support for Session Tickets"""

    ocspStapling: Optional[bool]
    """True if OCSP stapling is deployed on the server"""

    staplingRevocationStatus: Optional[int]
    """Same as Cert.revocationStatus, but for the stapled OCSP response."""

    staplingRevocationErrorMessage: Optional[str]
    """Description of the problem with the stapled OCSP response, if any."""

    sniRequired: Optional[bool]
    """If SNI support is required to access the web site."""

    httpStatusCode: Optional[int]
    """
    Status code of the final HTTP response seen. When submitting HTTP requests, redirections are followed, but only if they
    lead to the same hostname. If this field is not available, that means the HTTP request failed.
    """

    httpForwarding: Optional[str]
    """Available on a server that responded with a redirection to some other hostname."""

    supportsRc4: Optional[bool]
    """True if the server supports at least one RC4 suite."""

    rc4WithModern: Optional[bool]
    """True if RC4 is used with modern clients."""

    rc4Only: Optional[bool]
    """True if only RC4 suites are supported."""

    forwardSecrecy: Optional[int]
    """Indicates support for Forward Secrecy"""

    supportsAead: Optional[bool]
    """True if the server supports at least one AEAD suite."""

    supportsCBC: Optional[bool]
    """True if the server supports at least one CBC suite."""

    protocolIntolerance: Optional[int]
    """Indicates protocol version intolerance issues"""

    miscIntolerance: Optional[int]
    """Indicates various other types of intolerance"""

    sims: Optional[SimDetailsData]
    """Instance of SimDetails."""

    heartbleed: Optional[bool]
    """True if the server is vulnerable to the Heartbleed attack."""

    heartbeat: Optional[bool]
    """True if the server supports the Heartbeat extension."""

    openSslCcs: Optional[int]
    """Results of the CVE-2014-0224 test"""

    openSSLLuckyMinus20: Optional[int]
    """Results of the CVE-2016-2107 test"""

    ticketbleed: Optional[int]
    """Results of the ticketbleed CVE-2016-9244 test"""

    bleichenbacher: Optional[int]
    """Results of the Return Of Bleichenbacher's Oracle Threat (ROBOT) test"""

    zombiePoodle: Optional[int]
    """Results of the Zombie POODLE test"""

    goldenDoodle: Optional[int]
    """Results of the GOLDENDOODLE test"""

    zeroLengthPaddingOracle: Optional[int]
    """Results of the 0-Length Padding Oracle (CVE-2019-1559) test"""

    sleepingPoodle: Optional[int]
    """Results of the Sleeping POODLE test"""

    poodle: Optional[bool]
    """True if the endpoint is vulnerable to POODLE"""

    poodleTls: Optional[int]
    """Results of the POODLE TLS test"""

    fallbackScsv: Optional[bool]
    """
    True if the server supports TLS_FALLBACK_SCSV, false if it doesn't. This field will not be available if the server's
    support for TLS_FALLBACK_SCSV can't be tested because it supports only one protocol version (e.g., only TLS 1.2).
    """

    freak: Optional[bool]
    """True if the server is vulnerable to the FREAK attack, meaning it supports 512-bit key exchange."""

    hasSct: Optional[int]
    """Information about the availability of certificate transparency information (embedded SCTs)"""

    dhPrimes: Optional[List[str]]
    """List of hex-encoded DH primes used by the server. Not present if the server doesn't support the DH key exchange."""

    dhUsesKnownPrimes: Optional[int]
    """Whether the server uses known DH primes. Not present if the server doesn't support the DH key exchange."""

    dhYsReuse: Optional[bool]
    """True if the DH ephemeral server value is reused. Not present if the server doesn't support the DH key exchange."""

    ecdhParameterReuse: Optional[bool]
    """True if the server reuses its ECDHE values"""

    logjam: Optional[bool]
    """True if the server uses DH parameters weaker than 1024 bits."""

    chaCha20Preference: Optional[bool]
    """
    True if the server takes into account client preferences when deciding if to use ChaCha20 suites. Will be deprecated in
    new version.
    """

    hstsPolicy: Optional[HstsPolicyData]
    """Server's HSTS policy. Experimental."""

    hstsPreloads: Optional[List[HstsPreloadData]]
    """Information about preloaded HSTS policies."""

    hpkpPolicy: Optional[HpkpPolicyData]
    """Server's HPKP policy."""

    hpkpRoPolicy: Optional[HpkpPolicyData]
    """Server's HPKP-RO policy."""

    staticPkpPolicy: Optional[StaticPkpPolicyData]
    """Server's SPKP policy."""

    httpTransactions: Optional[List[HttpTransactionData]]
    """An array of HttpTransaction objects."""

    drownHosts: Optional[List[DrownHostsData]]
    """List of DROWN hosts."""

    drownErrors: Optional[bool]
    """True if error occurred in the DROWN test."""

    drownVulnerable: Optional[bool]
    """True if server vulnerable to the DROWN attack."""

    implementsTLS13MandatoryCS: Optional[bool]
    """True if server supports mandatory TLS 1.3 cipher suite (TLS_AES_128_GCM_SHA256), null if TLS 1.3 not supported."""

    zeroRTTEnabled: Optional[int]
    """Results of the 0-RTT test. This test will only be performed if TLS 1.3 is enabled."""



from dacite import from_dict

from ..data.info import InfoData
from ._api import _Api


class Info(_Api):
    """General information about the ssllabs API.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#check-ssl-labs-availability
    """

    async def get(self) -> InfoData:
        """Get information.

        :raises httpx.ConnectTimeout: SSL Labs Servers don't respond.
        :raises httpx.HTTPStatusError: A client or server error response occured.
        :raises httpx.ReadTimeout: SSL Labs Servers don't respond.
        """
        r = await self._call("info")
        return from_dict(data_class=InfoData, data=r.json())



from dataclasses import dataclass
from typing import Optional


@dataclass
class HstsPreloadData:
    """Dataclass for HSTS Preload objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#hstspreload
    """

    source: str
    """Source name"""

    hostname: str
    """Name of the host"""

    status: str
    """preload status"""

    error: Optional[str]
    """Error message, when status is 'error'"""

    sourceTime: int
    """Time, as a Unix timestamp, when the preload database was retrieved"""



import asyncio
import logging

from httpx import ConnectTimeout, HTTPStatusError, ReadTimeout

from .api import Analyze, Info, RootCertsRaw, StatusCodes
from .data.host import HostData
from .data.info import InfoData
from .data.status_codes import StatusCodesData


class Ssllabs():
    """Highlevel methods to interact with the SSL Labs Assessment APIs."""

    def __init__(self):
        self._logger = logging.getLogger("ssllabs.Ssllabs")
        self._semaphore = asyncio.Semaphore(1)

    async def availability(self) -> bool:
        """
        Check the availability of the SSL Labs servers.

        See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#error-response-status-codes
        """
        i = Info()
        try:
            await i.get()
            self._logger.info("SSL Labs servers are up an running.")
            return True
        except (HTTPStatusError, ReadTimeout, ConnectTimeout) as ex:
            self._logger.error(ex)
            return False

    async def analyze(self, host: str, publish: bool = False, ignore_mismatch: bool = False) -> HostData:
        """
        Test a particular host with respect to the cool off and the maximum number of assessments.

        :param host: Host to test
        :param publish: True if assessment results should be published on the public results boards
        :param ignore_mismatch: True if assessment shall proceed even when the server certificate doesn't match the hostname

        See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#protocol-usage
        """
        await self._semaphore.acquire()
        self._logger.info("Analyzing %s", host)
        i = Info()
        info = await i.get()

        # Wait for a free slot, if all slots are in use
        while info.currentAssessments >= info.maxAssessments:
            self._logger.warning("Already %i assessments running. Need to wait.", info.currentAssessments)
            await asyncio.sleep(1)
            info = await i.get()

        # If there is already an assessment running, wait the needed cool off until starting the next one
        if info.currentAssessments != 0:
            await asyncio.sleep(info.newAssessmentCoolOff / 1000)

        a = Analyze()
        host_object = await a.get(host=host,
                                  startNew="on",
                                  publish="on" if publish else "off",
                                  ignoreMismatch="on" if ignore_mismatch else "off")
        self._semaphore.release()
        while host_object.status not in ["READY", "ERROR"]:
            self._logger.debug("Assessment of %s not ready yet.", host)
            await asyncio.sleep(10)
            host_object = await a.get(host=host, all="done")
        return host_object

    async def info(self) -> InfoData:
        """
        Retrieve the engine and criteria version, and initialize the maximum number of concurrent assessments.

        See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#info
        """
        i = Info()
        return await i.get()

    async def root_certs(self, trust_store: int = 1) -> str:
        """
        Retrieve root certificates.

        :param trust_store: Trust store to return (1-Mozilla, 2-Apple MacOS, 3-Android, 4-Java, 5-Windows)

        See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#retrieve-root-certificates
        """
        if not 1 <= trust_store <= 5:
            raise ValueError("""Trust store not found. Please choose on of the following:
            1-Mozilla, 2-Apple MacOS, 3-Android, 4-Java, 5-Windows""")
        rcr = RootCertsRaw()
        return await rcr.get(trustStore=trust_store)

    async def status_codes(self) -> StatusCodesData:
        """
        Retrieve known status codes.

        See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#retrieve-known-status-codes
        """
        sc = StatusCodes()
        return await sc.get()



from dataclasses import dataclass
from typing import Dict, List, Optional


@dataclass
class HttpTransactionData:
    """Dataclass for HTTP Transaction objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#httptransaction
    """

    requestUrl: str
    """Request URL"""

    statusCode: Optional[int]
    """Response status code"""

    requestLine: Optional[str]
    """The entire request line as a single field"""

    requestHeaders: List[str]
    """An array of request HTTP headers, each with name and value"""

    responseLine: Optional[str]
    """The entire response line as a single field"""

    responseHeadersRaw: List[str]
    """All response headers as a single field (useful if the headers are malformed)"""

    responseHeaders: List[Dict]
    """An array of response HTTP headers, each with name and value"""

    fragileServer: bool
    """True if the server crashes when inspected by SSL Labs (in which case the full test is refused)"""



from dataclasses import dataclass


@dataclass
class NamedGroupData:
    """Dataclass for NamedGroup objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#namedgroup
    """

    id: int
    """Named curve ID"""

    name: str
    """named curve name"""

    bits: int
    """Named curve strength in EC bits"""



from dataclasses import dataclass
from typing import Dict, Optional


@dataclass
class HstsPolicyData:
    """Dataclass for HSTS Policy objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#hstspolicy
    """

    LONG_MAX_AGE: int
    """This constant contains what SSL Labs considers to be sufficiently large max-age value"""

    header: Optional[str]
    """The contents of the HSTS response header, if present"""

    status: str
    """HSTS status"""

    error: Optional[str]
    """Error message when error is encountered, null otherwise"""

    maxAge: Optional[int]
    """The max-age value specified in the policy; null if policy is missing or invalid or on parsing error"""

    includeSubDomains: Optional[bool]
    """True if the includeSubDomains directive is set; null otherwise"""

    preload: Optional[bool]
    """True if the preload directive is set; null otherwise"""

    directives: Optional[Dict]
    """List of raw policy directives"""



from dataclasses import dataclass
from typing import Optional


@dataclass
class SimClientData:
    """Dataclass for Simulation Client objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#simclient
    """

    id: int
    """Unique client ID."""

    name: str
    """Name of the client (e.g., Chrome)."""

    platform: Optional[str]
    """Name of the platform (e.g., XP SP3)."""

    version: str
    """Version of the software being simulated (e.g., 49)"""

    isReference: bool
    """
    True if the browser is considered representative of modern browsers, false otherwise. This flag does not correlate to
    client's capabilities, but is used by SSL Labs to determine if a particular configuration is effective. For example, to
    track Forward Secrecy support, we mark several representative browsers as "modern" and then test to see if they succeed in
    negotiating a FS suite. Just as an illustration, modern browsers are currently Chrome, Firefox (not ESR versions),
    IE/Win7, and Safari.
    """



from dataclasses import dataclass


@dataclass
class CaaRecordData:
    """Dataclass for CAA Record objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#caarecord
    """

    tag: str
    """A property of the CAA record"""

    value: str
    """Corresponding value of a CAA property"""

    flags: int
    """Corresponding flags of CAA property (8 bit)"""



from ._api import _Api


class RootCertsRaw(_Api):
    """Retrieve root certificates.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#retrieve-root-certificates
    """

    async def get(self, **kwargs) -> str:
        """Retrieve root certificates.

        :key trustStore: 1-Mozilla(default), 2-Apple MacOS, 3-Android, 4-Java, 5-Windows
        :raises httpx.ConnectTimeout: SSL Labs Servers don't respond.
        :raises httpx.HTTPStatusError: A client or server error response occured.
        :raises httpx.ReadTimeout: SSL Labs Servers don't respond.
        """
        self._verify_kwargs(kwargs.keys(), ["trustStore"])
        r = await self._call("getRootCertsRaw", **kwargs)
        return r.text



from dataclasses import dataclass
from typing import Optional

from .sim_client import SimClientData


@dataclass
class SimulationData:
    """Dataclass for Simulation objects.

    See also: https://github.com/ssllabs/ssllabs-scan/blob/master/ssllabs-api-docs-v3.md#simulation
    """

    client: SimClientData
    """Instance of SimClient."""

    errorCode: int
    """Zero if handshake was successful, 1 if it was not."""

    errorMessage: Optional[str]
    """Error message if simulation has failed."""

    attempts: int
    """Always 1 with the current implementation."""

    certChainId: Optional[str]
    """ID of the certificate chain."""

    protocolId: Optional[int]
    """Negotiated protocol ID."""

    suiteId: Optional[int]
    """Negotiated suite ID."""

    suiteName: Optional[str]
    """Negotiated suite Name."""

    kxType: Optional[str]
    """Negotiated key exchange, for example 'ECDH'."""

    kxStrength: Optional[int]
    """Negotiated key exchange strength, in RSA-equivalent bits."""

    dhBits: Optional[int]
    """Strength of DH params (e.g., 1024)"""

    dhP: Optional[int]
    """DH params, p component"""

    dhG: Optional[int]
    """DH params, g component"""

    dhYs: Optional[int]
    """DH params, Ys component"""

    namedGroupBits: Optional[int]
    """When ECDHE is negotiated, length of EC parameters."""

    namedGroupId: Optional[int]
    """When ECDHE is negotiated, EC curve ID."""

    namedGroupName: Optional[str]
    """When ECDHE is negotiated, EC curve nanme (e.g., 'secp256r1')."""

    keyAlg: Optional[str]
    """Connection certificate key algorithsm (e.g., 'RSA')."""

    keySize: Optional[int]
    """Connection certificate key size (e.g., 2048)."""

    sigAlg: Optional[str]
    """Connection certificate signature algorithm (e.g, 'SHA256withRSA')."""
